@@ -18,7 +18,7 @@
  *
  * May, 2000. Implemented on FreeBSD 4.0-RELEASE (Compaq Armada M700)
  *
- $Id: battstat_applet.c,v 1.38 2002/05/14 01:50:01 kevinv Exp $
+ $Id: battstat_applet.c,v 1.39 2002/05/20 20:41:51 kevinv Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -841,7 +841,6 @@ about_cb (PanelApplet *widget, gpointer data)
    	gdk_pixbuf_unref (pixbuf);
    
    gtk_window_set_wmclass (GTK_WINDOW (about_box), "battery charge monitor", "Batter Charge Monitor");
-   gnome_window_icon_set_from_file (GTK_WINDOW (about_box), GNOME_ICONDIR"/battstat.png");
    gtk_widget_show (about_box);
 }
 
@@ -1438,6 +1437,8 @@ battstat_applet_fill (PanelApplet *applet)
 
   if (DEBUG) g_print("main()\n");
   
+  gnome_window_icon_set_default_from_file (GNOME_ICONDIR"/battstat.png");
+  
   panel_applet_add_preferences (applet, "/schemas/apps/battstat-applet/prefs", NULL);
 
   apm_readinfo();
