@@ -1,4 +1,4 @@
-/* $Id: gweather-dialog.c,v 1.18 2002/03/26 01:44:04 kevinv Exp $ */
+/* $Id: gweather-dialog.c,v 1.21 2002/04/28 16:51:17 kevinv Exp $ */
 
 /*
  *  Papadimitriou Spiros <spapadim+@cs.cmu.edu>
@@ -76,6 +76,8 @@ void gweather_dialog_create (GWeatherApplet *gw_applet)
 						  GTK_DIALOG_DESTROY_WITH_PARENT,
 						  GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
 						  NULL);
+  gtk_dialog_set_default_response (GTK_DIALOG (gw_applet->gweather_dialog), 
+  				   GTK_RESPONSE_CLOSE);
 
   if (gw_applet->gweather_pref.radar_enabled)
       gtk_widget_set_usize (gw_applet->gweather_dialog, 570, 440);
@@ -274,7 +276,7 @@ void gweather_dialog_create (GWeatherApplet *gw_applet)
   gtk_widget_show (cond_vsep);
   gtk_box_pack_end (GTK_BOX (cond_hbox), cond_vsep, FALSE, FALSE, 0);
 
-  current_note_lbl = gtk_label_new (_("Current conditions"));
+  current_note_lbl = gtk_label_new_with_mnemonic (_("C_urrent conditions"));
   gtk_widget_show (current_note_lbl);
   gtk_notebook_set_tab_label (GTK_NOTEBOOK (weather_notebook), gtk_notebook_get_nth_page (GTK_NOTEBOOK (weather_notebook), 0), current_note_lbl);
 
@@ -297,7 +299,7 @@ void gweather_dialog_create (GWeatherApplet *gw_applet)
 
   gtk_container_add (GTK_CONTAINER (weather_notebook), forecast_hbox);
 
-  forecast_note_lbl = gtk_label_new (_("Forecast"));
+  forecast_note_lbl = gtk_label_new_with_mnemonic (_("_Forecast"));
   gtk_widget_show (forecast_note_lbl);
   gtk_notebook_set_tab_label (GTK_NOTEBOOK (weather_notebook), gtk_notebook_get_nth_page (GTK_NOTEBOOK (weather_notebook), 1), forecast_note_lbl);
 
@@ -309,9 +311,6 @@ void gweather_dialog_create (GWeatherApplet *gw_applet)
       gtk_container_set_border_width (GTK_CONTAINER (radar_vbox), 6);
 
 
-      /* PUSH */
-      gtk_widget_push_colormap (gdk_rgb_get_cmap ());
-
       ebox = gtk_event_box_new ();
       gtk_widget_show (ebox);
       gtk_box_pack_start (GTK_BOX (radar_vbox), ebox, FALSE, FALSE, 0);
@@ -321,15 +320,11 @@ void gweather_dialog_create (GWeatherApplet *gw_applet)
       gtk_widget_show (gw_applet->radar_image);
       gtk_container_add (GTK_CONTAINER (ebox), gw_applet->radar_image);
 
-      /* POP */
-      gtk_widget_pop_colormap ();
-
-
       radar_link_alignment = gtk_alignment_new (0.5, 0.5, 0, 0);
       gtk_widget_show (radar_link_alignment);
       gtk_box_pack_start (GTK_BOX (radar_vbox), radar_link_alignment, FALSE, FALSE, 0);
 
-      radar_link_btn = gtk_button_new_with_label (_("Visit Weather.com"));
+      radar_link_btn = gtk_button_new_with_mnemonic (_("_Visit Weather.com"));
       set_access_namedesc (radar_link_btn, _("URL link Button"),                                                            _("Click to Enter Weather.com"));
       gtk_widget_set_usize(radar_link_btn, 450, -2);
       gtk_widget_show (radar_link_btn);
@@ -338,7 +333,7 @@ void gweather_dialog_create (GWeatherApplet *gw_applet)
       gtk_signal_connect (GTK_OBJECT (radar_link_btn), "clicked",
                           GTK_SIGNAL_FUNC (link_cb), NULL);
 
-      radar_note_lbl = gtk_label_new (_("Radar map"));
+      radar_note_lbl = gtk_label_new_with_mnemonic (_("_Radar map"));
       gtk_widget_show (radar_note_lbl);
       gtk_notebook_set_tab_label (GTK_NOTEBOOK (weather_notebook), gtk_notebook_get_nth_page (GTK_NOTEBOOK (weather_notebook), 2), radar_note_lbl);
   }
