@@ -1,4 +1,4 @@
-/* $Id: weather.c,v 1.36 2002/02/23 22:05:36 kevinv Exp $ */
+/* $Id: weather.c,v 1.50 2002/04/08 01:12:15 kevinv Exp $ */
 
 /*
  *  Papadimitriou Spiros <spapadim+@cs.cmu.edu>
@@ -38,14 +38,12 @@
 void
 close_cb (GnomeVFSAsyncHandle *handle, GnomeVFSResult result, gpointer data);
 
-static WeatherUnits weather_units = UNITS_IMPERIAL;
+/* FIXME: these global variables will cause conflicts when multiple
+** instances of the applets update at the same time
+*/
 static WeatherForecastType weather_forecast = FORECAST_STATE;
 static gboolean weather_radar = FALSE;
 
-static gchar *weather_proxy_url = NULL;
-static gchar *weather_proxy_user = NULL;
-static gchar *weather_proxy_passwd = NULL;
-
 #define DATA_SIZE 5000
 
 /* Unit conversions and names */
@@ -58,10 +56,10 @@ static gchar *weather_proxy_passwd = NULL;
 #define WINDSPEED_KPH_TO_KNOTS(kph)    ((kph) * 0.539967)
 #define WINDSPEED_UNIT_STR(units) (((units) == UNITS_IMPERIAL) ? _("knots") : _("kph"))
 
-#define PRESSURE_INCH_TO_MM(inch)   ((inch) * 25.4)
-#define PRESSURE_MM_TO_INCH(mm)     ((mm) * 0.03937)
+#define PRESSURE_INCH_TO_HPA(inch)   ((inch) * 33.86)
+#define PRESSURE_HPA_TO_INCH(mm)     ((mm) / 33.86)
 #define PRESSURE_MBAR_TO_INCH(mbar) ((mbar) * 0.02963742)
-#define PRESSURE_UNIT_STR(units) (((units) == UNITS_IMPERIAL) ? _("inHg") : _("mmHg"))
+#define PRESSURE_UNIT_STR(units) (((units) == UNITS_IMPERIAL) ? _("inHg") : _("hPa"))
 
 #define VISIBILITY_SM_TO_KM(sm)  ((sm) * 1.609344)
 #define VISIBILITY_KM_TO_SM(km)  ((km) * 0.621371)
@@ -73,14 +71,10 @@ WeatherLocation *weather_location_new (const gchar *name, const gchar *code, con
     WeatherLocation *location;
 
     location = g_new(WeatherLocation, 1);
-    strncpy(location->name, name, WEATHER_LOCATION_NAME_MAX_LEN);
-    location->name[WEATHER_LOCATION_NAME_MAX_LEN] = '\0';
-    strncpy(location->code, code, WEATHER_LOCATION_CODE_LEN);
-    location->code[WEATHER_LOCATION_CODE_LEN] = '\0';
-    strncpy(location->zone, zone, WEATHER_LOCATION_ZONE_LEN);
-    location->zone[WEATHER_LOCATION_ZONE_LEN] = '\0';
-    strncpy(location->radar, radar, WEATHER_LOCATION_RADAR_LEN);
-    location->radar[WEATHER_LOCATION_RADAR_LEN] = '\0';
+    location->name = g_strdup(name);
+    location->code = g_strdup(code);
+    location->zone = g_strdup(zone);
+    location->radar = g_strdup(radar);
 
     return location;
 }
@@ -88,13 +82,26 @@ WeatherLocation *weather_location_new (const gchar *name, const gchar *code, con
 WeatherLocation *weather_location_config_read (PanelApplet *applet)
 {
     WeatherLocation *location;
-
-	location = weather_location_new(
-			panel_applet_gconf_get_string(applet, "location0", NULL),
-			panel_applet_gconf_get_string(applet, "location1", NULL),
-			panel_applet_gconf_get_string(applet, "location2", NULL),
-			panel_applet_gconf_get_string(applet, "location3", NULL)
-		);
+    gchar *name, *code, *zone, *radar;
+    
+    name = panel_applet_gconf_get_string(applet, "location0", NULL);
+    if (!name)
+    	name = g_strdup ("Pittsburgh");
+    code = panel_applet_gconf_get_string(applet, "location1", NULL);
+    if (!code)
+    	code = g_strdup ("KPIT");
+    zone = panel_applet_gconf_get_string(applet, "location2", NULL);
+    if (!zone)
+    	zone = g_strdup ("PAZ021");
+    radar = panel_applet_gconf_get_string(applet, "location3", NULL);
+    if (!radar)
+        radar = g_strdup ("pit");
+        
+    location = weather_location_new(name, code, zone, radar);
+    g_free (name);
+    g_free (code);
+    g_free (zone);
+    g_free (radar);
 
     return location;
 }
@@ -116,6 +123,8 @@ void weather_location_free (WeatherLocation *location)
 
 gboolean weather_location_equal (const WeatherLocation *location1, const WeatherLocation *location2)
 {
+    if (!location1->code || !location2->code)
+        return 1;
     return (strcmp(location1->code, location2->code) == 0);
 }
 
@@ -219,20 +228,16 @@ const gchar *weather_conditions_string (WeatherConditions cond)
 
 /* Locals turned global to facilitate asynchronous HTTP requests */
 
-static WeatherInfoFunc request_cb = NULL;
-static WeatherInfo *request_info = NULL;
 
 static inline gboolean requests_init (WeatherInfoFunc cb, WeatherInfo *info)
 {
-    if (info->requests_pending || request_info || request_cb)
+    if (info->requests_pending)
         return FALSE;
 
     /*g_assert(!metar_handle && !iwin_handle && !wx_handle && !met_handle);*/
 
     info->requests_pending = TRUE;
-    request_cb = cb;
-    request_info = info;
-	
+    	
     return TRUE;
 }
 
@@ -253,12 +258,7 @@ static inline void requests_done_check (WeatherInfo *info)
     if (!info->metar_handle && !info->iwin_handle && 
         !info->wx_handle && !info->met_handle) {
         info->requests_pending = FALSE;
-        /* Next two lines temporarily here */
-        if (weather_units == UNITS_METRIC)
-            weather_info_to_metric(request_info);
-        (*request_cb)(request_info);
-        request_cb = NULL;
-        request_info = NULL;
+        update_finish(info);
     }
 }
 
@@ -781,16 +781,15 @@ static void metar_finish_read(GnomeVFSAsyncHandle *handle, GnomeVFSResult result
     WeatherLocation *loc;
     gchar *metar, *eoln, *body, *temp;
     gboolean success = FALSE;
-    gchar searchkey[WEATHER_LOCATION_CODE_LEN + 2];
+    gchar *searchkey;
 
-    info->forecast = NULL;
     loc = info->location;
     body = (gchar *)buffer;
 
     body[body_len] = '\0';
 
     if (info->metar_buffer == NULL)
-      info->metar_buffer = g_strdup(body);
+    	info->metar_buffer = g_strdup(body);
     else
     {
 	temp = g_strdup(info->metar_buffer);
@@ -802,9 +801,10 @@ static void metar_finish_read(GnomeVFSAsyncHandle *handle, GnomeVFSResult result
     if (result == GNOME_VFS_ERROR_EOF)
     {
 
-  	g_snprintf (searchkey, sizeof (searchkey), "\n%s", loc->code);
+        searchkey = g_strdup_printf("\n%s", loc->code);
 
         metar = strstr(info->metar_buffer, searchkey);
+        g_free (searchkey);
         if (metar == NULL) {
             success = FALSE;
         } else {
@@ -818,9 +818,9 @@ static void metar_finish_read(GnomeVFSAsyncHandle *handle, GnomeVFSResult result
         }
 
         info->valid = success;
-	}
-	else if (result != GNOME_VFS_OK) {
-	    g_print("%s", gnome_vfs_result_to_string(result));
+    }
+    else if (result != GNOME_VFS_OK) {
+	g_print("%s", gnome_vfs_result_to_string(result));
         g_warning(_("Failed to get METAR data.\n"));
     } else {
 	gnome_vfs_async_read(handle, body, DATA_SIZE - 1, metar_finish_read, info);
@@ -829,7 +829,7 @@ static void metar_finish_read(GnomeVFSAsyncHandle *handle, GnomeVFSResult result
     }
     
     request_done(info->metar_handle, info);
-    
+    g_free (buffer);
     return;
 }
 
@@ -841,21 +841,20 @@ static void metar_finish_open (GnomeVFSAsyncHandle *handle, GnomeVFSResult resul
     int body_len;
     gchar *metar, *eoln;
     gboolean success = FALSE;
-    gchar searchkey[WEATHER_LOCATION_CODE_LEN + 2];
 
     g_return_if_fail(handle == info->metar_handle);
 
     g_return_if_fail(info != NULL);
-    
+   
     body = g_malloc0(DATA_SIZE);
     
     info->metar_buffer = NULL;
-    info->forecast = NULL;
     loc = info->location;
     if (loc == NULL) {
 	    g_warning (_("WeatherInfo missing location"));
 	    request_done(info->metar_handle, info);
 	    requests_done_check(info);
+	    g_free (body);
 	    return;
     }
 
@@ -863,10 +862,10 @@ static void metar_finish_open (GnomeVFSAsyncHandle *handle, GnomeVFSResult resul
         g_warning(_("Failed to get METAR data.\n"));
         info->metar_handle = NULL;
 	requests_done_check(info); 
+	g_free (body);
     } else {
         gnome_vfs_async_read(handle, body, DATA_SIZE - 1, metar_finish_read, info);
-	}
-
+    }
     return;
 }
 
@@ -998,15 +997,15 @@ static gchar *iwin_parse (gchar *iwin, WeatherLocation *loc)
         if (iwin_range_match(rangep, loc)) 
         {
             break;
-		}
+	}
     }
 
     if (ret != REG_NOMATCH) {
-        gchar *end = strstr(p, "\n</PRE>");
+        /*gchar *end = strstr(p, "\n</PRE>");
         if ((regexec(&iwin_re, p, 1, match, 0) != REG_NOMATCH) &&
             (end - p > match[0].rm_so))
             end = p + match[0].rm_so - 1;
-        *end = 0;
+        *end = 0;*/
         return g_strdup(rangep);
     } else {
         return NULL;
@@ -1021,8 +1020,7 @@ static void iwin_finish_read(GnomeVFSAsyncHandle *handle, GnomeVFSResult result,
     WeatherInfo *info = (WeatherInfo *)data;
     WeatherLocation *loc;
     gchar *body, *temp;
-    gchar *forecast;
-
+    
     info->forecast = NULL;
     loc = info->location;
     body = (gchar *)buffer;
@@ -1040,8 +1038,7 @@ static void iwin_finish_read(GnomeVFSAsyncHandle *handle, GnomeVFSResult result,
 	
     if (result == GNOME_VFS_ERROR_EOF)
     {
-	forecast = iwin_parse(info->iwin_buffer, loc);
-	info->forecast = forecast;
+        info->forecast = g_strdup (info->iwin_buffer);
     }
     else if (result != GNOME_VFS_OK) {
 	g_print("%s", gnome_vfs_result_to_string(result));
@@ -1053,7 +1050,7 @@ static void iwin_finish_read(GnomeVFSAsyncHandle *handle, GnomeVFSResult result,
     }
     
     request_done(info->iwin_handle, info);
-    
+   
     return;
 }
 
@@ -1069,11 +1066,11 @@ static void iwin_finish_open (GnomeVFSAsyncHandle *handle, GnomeVFSResult result
 	
     g_return_if_fail(info != NULL);
 
-	body = g_malloc0(DATA_SIZE);
+    body = g_malloc0(DATA_SIZE);
 
-	info->iwin_buffer = NULL;	
+    info->iwin_buffer = NULL;	
     if (info->forecast)
-        g_free (info->forecast);
+    g_free (info->forecast);
     info->forecast = NULL;
     loc = info->location;
     if (loc == NULL) {
@@ -1081,18 +1078,20 @@ static void iwin_finish_open (GnomeVFSAsyncHandle *handle, GnomeVFSResult result
 	    request_done(info->iwin_handle, info);
 	    info->iwin_handle = NULL;
 	    requests_done_check(info);
+	    g_free (body);
 	    return;
     }
 
     if (result != GNOME_VFS_OK) {
-        g_warning(_("Failed to get IWIN forecast data.\n"));
+        /* forecast data is not really interesting anyway ;) */
+	  g_warning(_("Failed to get IWIN forecast data.\n")); 
         info->iwin_handle = NULL;
         requests_done_check (info);
+        g_free (body);
     } else {
         gnome_vfs_async_read(handle, body, DATA_SIZE - 1, iwin_finish_read, info);
-	}
-	
-	return;
+    }
+    return;
 }
 
 static char *met_reprocess(char *x, int len)
@@ -1256,45 +1255,44 @@ static void met_finish_read(GnomeVFSAsyncHandle *handle, GnomeVFSResult result,
 			    gpointer buffer, GnomeVFSFileSize requested, 
 			    GnomeVFSFileSize body_len, gpointer data)
 {
-	WeatherInfo *info = (WeatherInfo *)data;
+    WeatherInfo *info = (WeatherInfo *)data;
     WeatherLocation *loc;
     gchar *body, *forecast, *temp;
-g_print ("read met \n");
-	info->forecast = NULL;
+
+    info->forecast = NULL;
     loc = info->location;
-	body = (gchar *)buffer;
-	body[body_len] = '\0';
+    body = (gchar *)buffer;
+    body[body_len] = '\0';
 	
-	if (info->met_buffer == NULL)
-		info->met_buffer = g_strdup(body);
-	else
-	{
-		temp = g_strdup(info->met_buffer);
-		g_free(info->met_buffer);
-		info->met_buffer = g_strdup_printf("%s%s", temp, body);
-		g_free(temp);
-	}
+    if (info->met_buffer == NULL)
+        info->met_buffer = g_strdup(body);
+    else
+    {
+        temp = g_strdup(info->met_buffer);
+	g_free(info->met_buffer);
+	info->met_buffer = g_strdup_printf("%s%s", temp, body);
+	g_free(temp);
+    }
 	
-	if (result == GNOME_VFS_ERROR_EOF)
-	{
-		g_print ("%s \n", info->met_buffer);
-		forecast = met_parse(info->met_buffer, loc);
+    if (result == GNOME_VFS_ERROR_EOF)
+    {
+	forecast = met_parse(info->met_buffer, loc);
         info->forecast = forecast;
     }
-	else if (result != GNOME_VFS_OK) {
-		g_print("%s", gnome_vfs_result_to_string(result));
-		info->met_handle = NULL;
-		requests_done_check (info);
+    else if (result != GNOME_VFS_OK) {
+	g_print("%s", gnome_vfs_result_to_string(result));
+	info->met_handle = NULL;
+	requests_done_check (info);
         g_warning(_("Failed to get Met Office data.\n"));
     } else {
-		gnome_vfs_async_read(handle, body, DATA_SIZE - 1, met_finish_read, info);
+	gnome_vfs_async_read(handle, body, DATA_SIZE - 1, met_finish_read, info);
 
-		return;
+	return;
     }
     
     request_done(info->met_handle, info);
-    
-	return;
+    g_free (buffer);
+    return;
 }
 
 static void met_finish_open (GnomeVFSAsyncHandle *handle, GnomeVFSResult result, gpointer data)
@@ -1308,10 +1306,10 @@ static void met_finish_open (GnomeVFSAsyncHandle *handle, GnomeVFSResult result,
     g_return_if_fail(handle == info->met_handle);
 
     g_return_if_fail(info != NULL);
-g_print ("finish open \n");
-	body = g_malloc0(DATA_SIZE);
 
-	info->met_buffer = NULL;	
+    body = g_malloc0(DATA_SIZE);
+
+    info->met_buffer = NULL;	
     info->forecast = NULL;
     loc = info->location;
     g_return_if_fail(loc != NULL);
@@ -1320,11 +1318,11 @@ g_print ("finish open \n");
         g_warning(_("Failed to get Met Office forecast data.\n"));
         info->met_handle = NULL;
         requests_done_check (info);
+        g_free (body);
     } else {
-		gnome_vfs_async_read(handle, body, DATA_SIZE - 1, met_finish_read, info);
-	}
-
-	return;
+    	gnome_vfs_async_read(handle, body, DATA_SIZE - 1, met_finish_read, info);
+    }
+    return;
 }
 
 static void metoffice_start_open (WeatherInfo *info)
@@ -1334,7 +1332,7 @@ static void metoffice_start_open (WeatherInfo *info)
     loc = info->location;
   
     url = g_strdup_printf("http://www.metoffice.gov.uk/datafiles/%s.html", loc->zone+1);
-g_print ("%s \n", url);
+
     gnome_vfs_async_open(&info->met_handle, url, GNOME_VFS_OPEN_READ, 
     			 0, met_finish_open, info);
     g_free(url);
@@ -1345,8 +1343,7 @@ g_print ("%s \n", url);
 /* Get forecast into newly alloc'ed string */
 static void iwin_start_open (WeatherInfo *info)
 {
-    gchar state[WEATHER_LOCATION_ZONE_LEN];
-    gchar *url;
+    gchar *url, *state, *zone;
     WeatherLocation *loc;
 
     g_return_if_fail(info != NULL);
@@ -1362,89 +1359,83 @@ static void iwin_start_open (WeatherInfo *info)
     	metoffice_start_open (info);
     	return;
     }
-
-    strncpy(state, loc->zone, 2);
-    state[2] = 0;
+    
+#if 0
     if (weather_forecast == FORECAST_ZONE)
-        url = g_strdup_printf("http://iwin.nws.noaa.gov/iwin/%s/zone.html", state);
+        url = g_strdup_printf("http://iwin.nws.noaa.gov/iwin/%s/zone.html",
+			loc->zone);
     else
-        url = g_strdup_printf("http://iwin.nws.noaa.gov/iwin/%s/state.html", state);
+        url = g_strdup_printf("http://iwin.nws.noaa.gov/iwin/%s/state.html",
+			loc->zone);
+#endif
+    
+    /* The zone for Pittsburgh (for example) is given as PAZ021 in the locations
+    ** file (the PA stands for the state pennsylvania). The url used wants the state
+    ** as pa, and the zone as lower case paz021.
+    */
+    zone = g_strdup (loc->zone);
+    g_strdown (zone);
+    state = g_strdup (zone);
+    state[2] = '\0';
+    
+    url = g_strdup_printf ("http://weather.noaa.gov/pub/data/forecasts/zone/%s/%s.txt",
+        		   state, zone); 
+    if (zone)
+    	g_free (zone);   
+    if (state)
+    	g_free (state);
+
     gnome_vfs_async_open(&info->iwin_handle, url, GNOME_VFS_OPEN_READ, 
     			 0, iwin_finish_open, info);
     g_free(url);
 
 }
 
-static GdkPixmap *wx_construct (gpointer data, gint data_len)
-{
-    GdkPixmap *pixmap;
-    GdkPixbuf *pixbuf;
-    GdkPixbufLoader *pixbuf_loader;
-
-    pixbuf_loader = gdk_pixbuf_loader_new ();
-
-    gdk_pixbuf_loader_write (pixbuf_loader, data, data_len, NULL);
-    gdk_pixbuf_loader_close (pixbuf_loader, NULL);
-
-    pixbuf = gdk_pixbuf_loader_get_pixbuf (pixbuf_loader);
-
-    pixmap = NULL;
-
-    if (pixbuf != NULL) {
-	    gdk_pixbuf_render_pixmap_and_mask (pixbuf, &pixmap, NULL, 127);
-	    g_print ("pixbud != NULL\n");
-    }
-
-    g_object_unref (G_OBJECT (pixbuf_loader));
-
-    return pixmap;
-}
-
 static void wx_finish_read(GnomeVFSAsyncHandle *handle, GnomeVFSResult result, 
 			   gpointer buffer, GnomeVFSFileSize requested, 
 			   GnomeVFSFileSize body_len, gpointer data)
 {
     WeatherInfo *info = (WeatherInfo *)data;
     WeatherLocation *loc;
-    gchar *body, *temp;
     GdkPixmap *pixmap = NULL;
 	
-	info->forecast = NULL;
-	info->radar = NULL;
+    info->radar = NULL;
     loc = info->location;
-	body = (gchar *)buffer;
-	body[body_len] = '\0';
 
-	if (info->radar_buffer == NULL)
-		info->radar_buffer = g_strdup(body);
-	else
-	{
-		temp = g_strdup(info->radar_buffer);
-		g_free(info->radar_buffer);
-		info->radar_buffer = g_strdup_printf("%s%s", temp, body);
-		g_free(temp);
-	}
-	
-	if (result == GNOME_VFS_ERROR_EOF)
-	{
-		pixmap = wx_construct (info->radar_buffer, strlen (info->radar_buffer));
-g_print ("got radar \n");
-        info->radar = pixmap;
-	}
-	else if (result != GNOME_VFS_OK) {
-		g_print("%s", gnome_vfs_result_to_string(result));
+    if (result == GNOME_VFS_OK && body_len != 0) {
+        GError *error = NULL;
+    	gdk_pixbuf_loader_write (info->radar_loader, buffer, body_len, &error);
+        if (error) {
+            g_print ("%s \n", error->message);
+            g_error_free (error);
+        }
+        gnome_vfs_async_read(handle, buffer, DATA_SIZE - 1, wx_finish_read, info);
+        return;
+    }
+    else if (result == GNOME_VFS_ERROR_EOF)
+    {
+        GdkPixbuf *pixbuf;
+    	
+        gdk_pixbuf_loader_close (info->radar_loader, NULL);
+        pixbuf = gdk_pixbuf_loader_get_pixbuf (info->radar_loader);
+
+        if (pixbuf != NULL) {
+            if (info->radar)
+                g_object_unref (info->radar);
+	    gdk_pixbuf_render_pixmap_and_mask (pixbuf, &info->radar, NULL, 127);
+        }
+    }
+    else
+    {
+        g_print("%s", gnome_vfs_result_to_string(result));
         g_warning(_("Failed to get METAR data.\n"));
         info->wx_handle = NULL;
         requests_done_check (info);
-    } else {
-		gnome_vfs_async_read(handle, body, DATA_SIZE - 1, wx_finish_read, info);
-
-		return;
     }
-
-	request_done(info->wx_handle, info);
-	    
-	return;
+    
+    request_done(info->wx_handle, info);
+    g_free (buffer);    
+    return;
 }
 
 static void wx_finish_open (GnomeVFSAsyncHandle *handle, GnomeVFSResult result, gpointer data)
@@ -1470,8 +1461,10 @@ static void wx_finish_open (GnomeVFSAsyncHandle *handle, GnomeVFSResult result,
         g_warning(_("Failed to get radar map image.\n"));
         info->wx_handle = NULL;
         requests_done_check (info);
+        g_free (body);
     } else {
-    	gnome_vfs_async_read(handle, body, DATA_SIZE -1, wx_finish_read, info);
+        gnome_vfs_async_read(handle, body, DATA_SIZE -1, wx_finish_read, info); 
+    	
     }
      return;
 }
@@ -1481,25 +1474,30 @@ static void wx_start_open (WeatherInfo *info)
 {
     gchar *url;
     WeatherLocation *loc;
-	
+    
     g_return_if_fail(info != NULL);
     info->radar = NULL;
+    info->radar_loader = gdk_pixbuf_loader_new ();
     loc = info->location;
     g_return_if_fail(loc != NULL);
 
     if (loc->radar[0] == '-')
         return;
-
-    url = g_strdup_printf("http://image.weather.com/images/radar/single_site/%sloc_450x284.jpg", loc->radar);
-g_print ("url %s \n", url);
+    if (info->radar_url)
+    	url = g_strdup (info->radar_url);
+    else {
+	 url = g_strdup_printf ("http://image.weather.com/web/radar/us_%s_closeradar_medium_usen.jpg", loc->radar);
+    }
+ 
     gnome_vfs_async_open(&info->wx_handle, url, GNOME_VFS_OPEN_READ, 
     			 0, wx_finish_open, info);
+    				 
     g_free(url);
 
     return;
 }
 
-gboolean _weather_info_fill (WeatherInfo *info, WeatherLocation *location, WeatherInfoFunc cb)
+gboolean _weather_info_fill (GWeatherApplet *applet, WeatherInfo *info, WeatherLocation *location, WeatherInfoFunc cb)
 {
     g_return_val_if_fail(((info == NULL) && (location != NULL)) || \
                          ((info != NULL) && (location == NULL)), FALSE);
@@ -1523,10 +1521,8 @@ gboolean _weather_info_fill (WeatherInfo *info, WeatherLocation *location, Weath
 		}
     }
 
-    /* FIX */
-    
+    /* Update in progress */
     if (!requests_init(cb, info)) {
-        g_warning(_("Another update already in progress!\n"));
         return FALSE;
     }
 
@@ -1546,11 +1542,16 @@ gboolean _weather_info_fill (WeatherInfo *info, WeatherLocation *location, Weath
     info->visibility = -1.0;
     info->forecast = NULL;
     info->radar = NULL;
+    info->radar_url = NULL;
+    if (applet->gweather_pref.use_custom_radar_url && applet->gweather_pref.radar) {
+    	info->radar_url = g_strdup (applet->gweather_pref.radar); 
+    }   
     info->metar_handle = NULL;
     info->iwin_handle = NULL;
     info->wx_handle = NULL;
     info->met_handle = NULL;
     info->requests_pending = TRUE;
+    info->applet = applet;
 
     metar_start_open(info);
     iwin_start_open(info);
@@ -1589,7 +1590,7 @@ void weather_info_config_write (WeatherInfo *info)
 WeatherInfo *weather_info_config_read (PanelApplet *applet)
 {
     WeatherInfo *info = g_new(WeatherInfo, 1);
-/*
+#if 0
     info->valid = gnome_config_get_bool("valid=FALSE");
     info->location = weather_location_config_read("location");
     info->units = (WeatherUnits)gnome_config_get_int("units=0");
@@ -1607,8 +1608,8 @@ WeatherInfo *weather_info_config_read (PanelApplet *applet)
     info->visibility = gnome_config_get_float("visibility=0");
     info->forecast = gnome_config_get_string("forecast=None");
     info->radar = NULL;
-*/
-	info->valid = FALSE;
+#endif
+    info->valid = FALSE;
     info->location = weather_location_config_read(applet);
     info->units = (WeatherUnits)0;
     info->update = (WeatherUpdate)0;
@@ -1647,6 +1648,7 @@ WeatherInfo *weather_info_clone (const WeatherInfo *info)
     clone->location = weather_location_clone(info->location);
     /* This handles null correctly */
     clone->forecast = g_strdup(info->forecast);
+    clone->radar_url = g_strdup (info->radar_url); 
 
     clone->radar = info->radar;
     if (clone->radar != NULL)
@@ -1666,22 +1668,13 @@ void weather_info_free (WeatherInfo *info)
 		gdk_pixmap_unref (info->radar);
 		info->radar = NULL;
 	}
+	if (info->radar)
+	    g_free (info->radar);
 	g_free(info);
     }
     
 }
 
-
-void weather_units_set (WeatherUnits units)
-{
-    weather_units = units;
-}
-
-WeatherUnits weather_units_get (void)
-{
-    return weather_units;
-}
-
 void weather_forecast_set (WeatherForecastType forecast)
 {
     weather_forecast = forecast;
@@ -1702,21 +1695,6 @@ gboolean weather_radar_get (void)
     return weather_radar;
 }
 
-
-void weather_proxy_set (const gchar *url, const gchar *user, const gchar *passwd)
-{
-    g_free(weather_proxy_url);    weather_proxy_url = NULL;
-    g_free(weather_proxy_user);   weather_proxy_user = NULL;
-    g_free(weather_proxy_passwd); weather_proxy_passwd = NULL;
-
-    if (url && (strlen(url) > 0))
-        weather_proxy_url = g_strdup(url);
-    if (user && (strlen(user) > 0))
-        weather_proxy_user = g_strdup(user);
-    if (passwd && (strlen(passwd) > 0))
-        weather_proxy_passwd = g_strdup(passwd);
-}
-
 void weather_info_to_metric (WeatherInfo *info)
 {
     g_return_if_fail(info != NULL);
@@ -1728,7 +1706,7 @@ void weather_info_to_metric (WeatherInfo *info)
     info->temp = TEMP_F_TO_C(info->temp);
     info->dew = TEMP_F_TO_C(info->dew);
     info->windspeed = WINDSPEED_KNOTS_TO_KPH(info->windspeed);
-    info->pressure = PRESSURE_INCH_TO_MM(info->pressure);
+    info->pressure = PRESSURE_INCH_TO_HPA(info->pressure);
     info->visibility = VISIBILITY_SM_TO_KM(info->visibility);
 
     /* Change units flag */
@@ -1746,7 +1724,7 @@ void weather_info_to_imperial (WeatherInfo *info)
     info->temp = TEMP_C_TO_F(info->temp);
     info->dew = TEMP_C_TO_F(info->dew);
     info->windspeed = WINDSPEED_KPH_TO_KNOTS(info->windspeed);
-    info->pressure = PRESSURE_MM_TO_INCH(info->pressure);
+    info->pressure = PRESSURE_HPA_TO_INCH(info->pressure);
     info->visibility = VISIBILITY_KM_TO_SM(info->visibility);
 
     /* Change units flag */
@@ -1883,7 +1861,8 @@ GdkPixmap *weather_info_get_radar (WeatherInfo *info)
 const gchar *weather_info_get_temp_summary (WeatherInfo *info)
 {
     static gchar buf[10];
-    g_return_val_if_fail(info != NULL, NULL);
+    if (!info)
+        return NULL;
     if (!info->valid)
         return "--";
     g_snprintf(buf, sizeof (buf), "%d\302\260", (int)(info->temp + 0.5));
@@ -1904,10 +1883,8 @@ gchar *weather_info_get_weather_summary (WeatherInfo *info)
 }
 
 
-static GdkPixmap **weather_pixmaps_mini = NULL;
-static GdkBitmap **weather_pixmaps_mini_mask = NULL;
-static GdkPixmap **weather_pixmaps = NULL;
-static GdkBitmap **weather_pixmaps_mask = NULL;
+static GdkPixbuf **weather_pixbufs_mini = NULL;
+static GdkPixbuf **weather_pixbufs = NULL;
 
 #include "pixmaps/unknown-mini.xpm"
 #include "pixmaps/sun-mini.xpm"
@@ -1938,6 +1915,7 @@ static GdkBitmap **weather_pixmaps_mask = NULL;
 
 #define NUM_PIX       8
 
+#if 0
 static void
 xpm_to_pixmap (char **xpm_data, GdkPixmap **pixmap, GdkBitmap **mask)
 {
@@ -1952,8 +1930,9 @@ xpm_to_pixmap (char **xpm_data, GdkPixmap **pixmap, GdkBitmap **mask)
 		gdk_pixbuf_unref (pixbuf);
 	}
 }
+#endif
 
-static void init_pixmaps (void)
+static void init_pixbufs (void)
 {
     static gboolean initialized = FALSE;
 
@@ -1961,40 +1940,36 @@ static void init_pixmaps (void)
        return;
     initialized = TRUE;
 
-    weather_pixmaps_mini = g_new(GdkPixmap *, NUM_PIX);
-    weather_pixmaps_mini_mask = g_new(GdkBitmap *, NUM_PIX);
-    xpm_to_pixmap (unknown_mini_xpm, &weather_pixmaps_mini[PIX_UNKNOWN], &weather_pixmaps_mini_mask[PIX_UNKNOWN]);
-    xpm_to_pixmap (sun_mini_xpm, &weather_pixmaps_mini[PIX_SUN], &weather_pixmaps_mini_mask[PIX_SUN]);
-    xpm_to_pixmap (suncloud_mini_xpm, &weather_pixmaps_mini[PIX_SUNCLOUD], &weather_pixmaps_mini_mask[PIX_SUNCLOUD]);
-    xpm_to_pixmap (cloud_mini_xpm, &weather_pixmaps_mini[PIX_CLOUD], &weather_pixmaps_mini_mask[PIX_CLOUD]);
-    xpm_to_pixmap (rain_mini_xpm, &weather_pixmaps_mini[PIX_RAIN], &weather_pixmaps_mini_mask[PIX_RAIN]);
-    xpm_to_pixmap (tstorm_mini_xpm, &weather_pixmaps_mini[PIX_TSTORM], &weather_pixmaps_mini_mask[PIX_TSTORM]);
-    xpm_to_pixmap (snow_mini_xpm, &weather_pixmaps_mini[PIX_SNOW], &weather_pixmaps_mini_mask[PIX_SNOW]);
-    xpm_to_pixmap (fog_mini_xpm, &weather_pixmaps_mini[PIX_FOG], &weather_pixmaps_mini_mask[PIX_FOG]);
+    weather_pixbufs_mini = g_new(GdkPixbuf *, NUM_PIX);
+    weather_pixbufs_mini[PIX_UNKNOWN] = gdk_pixbuf_new_from_xpm_data ((const char **)unknown_mini_xpm);	
+    weather_pixbufs_mini[PIX_SUN] = gdk_pixbuf_new_from_xpm_data ((const char **)sun_mini_xpm);	
+    weather_pixbufs_mini[PIX_SUNCLOUD] = gdk_pixbuf_new_from_xpm_data ((const char **)suncloud_mini_xpm);	
+    weather_pixbufs_mini[PIX_CLOUD] = gdk_pixbuf_new_from_xpm_data ((const char **)cloud_mini_xpm);	
+    weather_pixbufs_mini[PIX_RAIN] = gdk_pixbuf_new_from_xpm_data ((const char **)rain_mini_xpm);	
+    weather_pixbufs_mini[PIX_TSTORM] = gdk_pixbuf_new_from_xpm_data ((const char **)tstorm_mini_xpm);	
+    weather_pixbufs_mini[PIX_SNOW] = gdk_pixbuf_new_from_xpm_data ((const char **)snow_mini_xpm);	
+    weather_pixbufs_mini[PIX_FOG] = gdk_pixbuf_new_from_xpm_data ((const char **)fog_mini_xpm);	
 
-    weather_pixmaps = g_new(GdkPixmap *, NUM_PIX);
-    weather_pixmaps_mask = g_new(GdkBitmap *, NUM_PIX);
-    xpm_to_pixmap (unknown_xpm, &weather_pixmaps[PIX_UNKNOWN], &weather_pixmaps_mask[PIX_UNKNOWN]);
-    xpm_to_pixmap (sun_xpm, &weather_pixmaps[PIX_SUN], &weather_pixmaps_mask[PIX_SUN]);
-    xpm_to_pixmap (suncloud_xpm, &weather_pixmaps[PIX_SUNCLOUD], &weather_pixmaps_mask[PIX_SUNCLOUD]);
-    xpm_to_pixmap (cloud_xpm, &weather_pixmaps[PIX_CLOUD], &weather_pixmaps_mask[PIX_CLOUD]);
-    xpm_to_pixmap (rain_xpm, &weather_pixmaps[PIX_RAIN], &weather_pixmaps_mask[PIX_RAIN]);
-    xpm_to_pixmap (tstorm_xpm, &weather_pixmaps[PIX_TSTORM], &weather_pixmaps_mask[PIX_TSTORM]);
-    xpm_to_pixmap (snow_xpm, &weather_pixmaps[PIX_SNOW], &weather_pixmaps_mask[PIX_SNOW]);
-    xpm_to_pixmap (fog_xpm, &weather_pixmaps[PIX_FOG], &weather_pixmaps_mask[PIX_FOG]);
+    weather_pixbufs = g_new(GdkPixbuf *, NUM_PIX);
+    weather_pixbufs[PIX_UNKNOWN] = gdk_pixbuf_new_from_xpm_data ((const char **)unknown_xpm);
+    weather_pixbufs[PIX_SUN] = gdk_pixbuf_new_from_xpm_data ((const char **)sun_xpm);
+    weather_pixbufs[PIX_SUNCLOUD] = gdk_pixbuf_new_from_xpm_data ((const char **)suncloud_xpm);
+    weather_pixbufs[PIX_CLOUD] = gdk_pixbuf_new_from_xpm_data ((const char **)cloud_xpm);
+    weather_pixbufs[PIX_RAIN] = gdk_pixbuf_new_from_xpm_data ((const char **)rain_xpm);
+    weather_pixbufs[PIX_TSTORM] = gdk_pixbuf_new_from_xpm_data ((const char **)tstorm_xpm);
+    weather_pixbufs[PIX_SNOW] = gdk_pixbuf_new_from_xpm_data ((const char **)snow_xpm);
+    weather_pixbufs[PIX_FOG] = gdk_pixbuf_new_from_xpm_data ((const char **)fog_xpm);
 }
 
-void _weather_info_get_pixmap (WeatherInfo *info, gboolean mini, GdkPixmap **pixmap, GdkBitmap **mask)
+void _weather_info_get_pixbuf (WeatherInfo *info, gboolean mini, GdkPixbuf **pixbuf)
 {
-    GdkPixmap **pixmaps;
-    GdkBitmap **masks;
+    GdkPixbuf **pixbufs;
     gint idx = -1;
 
-    g_return_if_fail(pixmap != NULL);
+    g_return_if_fail(pixbuf != NULL);
 
-    init_pixmaps();
-    pixmaps = mini ? weather_pixmaps_mini : weather_pixmaps;
-    masks = mini ? weather_pixmaps_mini_mask : weather_pixmaps_mask;
+    init_pixbufs();
+    pixbufs = mini ? weather_pixbufs_mini : weather_pixbufs;
 
     if (!info || !info->valid) {
         idx = PIX_UNKNOWN;
@@ -2089,9 +2064,7 @@ void _weather_info_get_pixmap (WeatherInfo *info, gboolean mini, GdkPixmap **pix
         }
     }
 
-    *pixmap = pixmaps[idx];
-    if (mask)
-        *mask = masks[idx];
+    *pixbuf = pixbufs[idx];
 }
 
 
