@@ -33,13 +33,101 @@
 #include "mini-commander_applet.h"
 
 static void reset_temporary_prefs(void);
+static void phelp_cb (void);
 
-static void
-entry_changed_signal(GtkWidget *entry_widget, char **data)
+static gboolean
+pattern_entry_changed_signal(GtkWidget *entry_widget, GdkEventFocus *event, gpointer data)
 {
-    free(*data);
-    *data = (char *) malloc(sizeof(char) * (strlen(gtk_entry_get_text(GTK_ENTRY(entry_widget))) + 1));
-    strcpy(*data, gtk_entry_get_text(GTK_ENTRY(entry_widget)));
+    PanelApplet *applet = g_object_get_data (G_OBJECT (entry_widget), "applet");
+    properties *prop = g_object_get_data (G_OBJECT (entry_widget), "prop");
+    GConfValue *macro_patterns;
+    GSList *pattern_list = NULL;
+    gchar *new;
+    gint i, current;
+        
+    new = gtk_editable_get_chars (GTK_EDITABLE (entry_widget), 0, -1);
+    current = GPOINTER_TO_INT (data);
+    if (!new)
+    	return FALSE;
+
+    if (!g_strcasecmp (new, prop->macro_pattern[current]))
+    	return FALSE;
+	 
+    free(prop->macro_pattern[current]);
+    prop->macro_pattern[current] = new;
+    
+    /* save the patterns */
+    for(i=0; i<=MAX_NUM_MACROS-1; i++) {
+        GConfValue *entry;
+	    
+	entry = gconf_value_new (GCONF_VALUE_STRING);
+	gconf_value_set_string (entry, prop->macro_pattern[i]);
+	pattern_list = g_slist_append (pattern_list, entry);	   
+    
+    }  
+    
+    macro_patterns = gconf_value_new (GCONF_VALUE_LIST);
+    if (pattern_list) {
+        gconf_value_set_list_type (macro_patterns, GCONF_VALUE_STRING);
+        gconf_value_set_list (macro_patterns, pattern_list);
+        panel_applet_gconf_set_value (applet, "macro_patterns", macro_patterns, NULL);               
+    }
+#if 0
+    if(prop->macro_regex[current] != NULL)
+	free(prop->macro_regex[current]);
+#endif
+    if(prop->macro_pattern[current] && prop->macro_pattern[current][0] != '\0')
+    {
+	prop->macro_regex[current] = malloc(sizeof(regex_t));
+	/* currently we don't care about syntax errors in regex patterns */
+	regcomp(prop->macro_regex[current], prop->macro_pattern[current], REG_EXTENDED);
+    }
+    else
+    {
+	prop->macro_regex[current] = NULL;
+    }
+    return FALSE;
+}
+
+static gboolean
+command_entry_changed_signal(GtkWidget *entry_widget, GdkEventFocus *event, gpointer data)
+{
+    PanelApplet *applet = g_object_get_data (G_OBJECT (entry_widget), "applet");
+    properties *prop = g_object_get_data (G_OBJECT (entry_widget), "prop");
+    GConfValue *macro_command;
+    GSList *command_list = NULL;
+    gchar *new;
+    gint i;
+     
+    new = gtk_editable_get_chars (GTK_EDITABLE (entry_widget), 0, -1);
+    i = GPOINTER_TO_INT (data);
+    if (!new)
+    	return FALSE;
+ 
+    if (!g_strcasecmp (new, prop->macro_command[i]))
+    	return FALSE;
+ 
+    free(prop->macro_command[i]);
+    prop->macro_command[i] = new;
+    
+    /* save the commands */
+    for(i=0; i<=MAX_NUM_MACROS-1; i++) {
+        GConfValue *entry;
+	    
+	entry = gconf_value_new (GCONF_VALUE_STRING);
+	gconf_value_set_string (entry, prop->macro_command[i]);
+	command_list = g_slist_append (command_list, entry);	   
+    
+    }  
+    
+    macro_command = gconf_value_new (GCONF_VALUE_LIST);
+    if (command_list) {
+        gconf_value_set_list_type (macro_command, GCONF_VALUE_STRING);
+        gconf_value_set_list (macro_command, command_list);
+        panel_applet_gconf_set_value (applet, "macro_commands", macro_command, NULL);               
+    }
+
+    return FALSE;
 }
 
 static void
@@ -55,12 +143,18 @@ color_cmd_fg_changed_signal(GtkWidget *color_picker_widget,
     MCData *mcdata = data;
     PanelApplet *applet = mcdata->applet;
     properties *prop;
+    gushort red, green, blue;
     
     prop = g_object_get_data (G_OBJECT (applet), "prop");
+    gnome_color_picker_get_i16 (GNOME_COLOR_PICKER (color_picker_widget),
+    				&red,
+    				&green,
+    				&blue,
+    				NULL);
     
-    prop->cmd_line_color_fg_r = (int) r;
-    prop->cmd_line_color_fg_g = (int) g; 
-    prop->cmd_line_color_fg_b = (int) b;
+    prop->cmd_line_color_fg_r = (int) red;
+    prop->cmd_line_color_fg_g = (int) green; 
+    prop->cmd_line_color_fg_b = (int) blue;
    
     command_entry_update_color (mcdata->entry, prop);
     
@@ -82,12 +176,18 @@ color_cmd_bg_changed_signal(GtkWidget *color_picker_widget,
     MCData *mcdata = data;
     PanelApplet *applet = mcdata->applet;
     properties *prop;
+    gushort red, green, blue;
     
     prop = g_object_get_data (G_OBJECT (applet), "prop");
+    gnome_color_picker_get_i16 (GNOME_COLOR_PICKER (color_picker_widget),
+    				&red,
+    				&green,
+    				&blue,
+    				NULL);
     
-    prop->cmd_line_color_bg_r = (int) r;
-    prop->cmd_line_color_bg_g = (int) g; 
-    prop->cmd_line_color_bg_b = (int) b;
+    prop->cmd_line_color_bg_r = (int) red;
+    prop->cmd_line_color_bg_g = (int) green;
+    prop->cmd_line_color_bg_b = (int) blue;
    
     command_entry_update_color (mcdata->entry, prop);
 
@@ -165,18 +265,21 @@ load_session(MCData *mcdata)
 {
     PanelApplet *applet = mcdata->applet;
     GConfValue *history;
+    GConfValue *macro_patterns, *macro_commands;
+    GSList *pattern_list = NULL, *command_list = NULL;
     properties *prop;
     int i;
     char section[MAX_COMMAND_LENGTH + 100];
     char default_macro_pattern[MAX_MACRO_PATTERN_LENGTH];
     char default_macro_command[MAX_COMMAND_LENGTH];
     
-    gnome_config_push_prefix("/mini-commander/"); 
-    
     prop = g_new0 (properties, 1);
     
     prop->show_time = FALSE;
     prop->show_date = FALSE;
+    prop->show_default_theme = panel_applet_gconf_get_bool (applet, 
+                                      			    "show_default_theme", 
+                                      		  	    NULL);
     prop->show_handle = panel_applet_gconf_get_bool (applet, "show_handle", NULL);
     prop->show_frame = panel_applet_gconf_get_bool(applet, "show_frame", NULL);
 
@@ -205,9 +308,47 @@ load_session(MCData *mcdata)
     							   NULL);
     prop->cmd_line_color_bg_b = panel_applet_gconf_get_int(applet, "cmd_line_color_bg_b",
     							   NULL);
-
-    /* macros */
-    for(i=0; i<=MAX_NUM_MACROS-1; i++)
+    macro_patterns = panel_applet_gconf_get_value (applet, "macro_patterns", NULL);
+    macro_commands = panel_applet_gconf_get_value (applet, "macro_commands", NULL);
+    
+    if (macro_patterns &&  macro_commands) {
+    	GSList *list1 = NULL, *list2 = NULL;
+    	gint i = 0;
+    	
+        list1 = gconf_value_get_list (macro_patterns);
+        list2 = gconf_value_get_list (macro_commands);
+        while (list1 && list2) {
+            GConfValue *value1 = list1->data;
+            GConfValue *value2 = list2->data;
+            const gchar *pattern = NULL, *command = NULL;
+            
+            pattern = gconf_value_get_string (value1);
+            command = gconf_value_get_string (value2);
+            
+            free(prop->macro_pattern[i]);
+	    prop->macro_pattern[i] = (char *)pattern;
+            free(prop->macro_command[i]);
+	    prop->macro_command[i] = (char *)command; 
+	    
+	    if(prop->macro_pattern[i][0] != '\0')
+		{
+		    prop->macro_regex[i] = malloc(sizeof(regex_t));
+		    /* currently we don't care about syntax errors in regex patterns */
+		    regcomp(prop->macro_regex[i], prop->macro_pattern[i], REG_EXTENDED);
+		}
+	    else
+		{
+		    prop->macro_regex[i] = NULL;
+		}
+		
+            list1 = g_slist_next (list1);
+            list2 = g_slist_next (list2);
+            i++;
+        }
+    }
+    else {    
+    /* defaults */
+    	for(i=0; i<=MAX_NUM_MACROS-1; i++)
 	{
 	    switch (i + 1) 
 		/* set default macros */
@@ -283,29 +424,49 @@ load_session(MCData *mcdata)
 		default:
 		    strcpy(default_macro_pattern, "");
 		    strcpy(default_macro_command, "");		   
-		}
+		} 
 
-	    g_snprintf(section, sizeof(section), "mini_commander/macro_pattern_%.2u=%s", i+1, default_macro_pattern);
-	    free(prop->macro_pattern[i]);
-	    prop->macro_pattern[i] = (char *)gnome_config_get_string((gchar *) section);
-
-	    g_snprintf(section, sizeof(section), "mini_commander/macro_command_%.2u=%s", i+1, default_macro_command);
-	    free(prop->macro_command[i]);
-	    prop->macro_command[i] = (char *)gnome_config_get_string((gchar *) section);
-
-	    if(prop->macro_pattern[i][0] != '\0')
+		free(prop->macro_pattern[i]);
+	        prop->macro_pattern[i] = g_strdup (default_macro_pattern);
+            	free(prop->macro_command[i]);
+	    	prop->macro_command[i] = g_strdup (default_macro_command); 
+	    
+	    	if(prop->macro_pattern[i][0] != '\0')
 		{
 		    prop->macro_regex[i] = malloc(sizeof(regex_t));
 		    /* currently we don't care about syntax errors in regex patterns */
 		    regcomp(prop->macro_regex[i], prop->macro_pattern[i], REG_EXTENDED);
 		}
-	    else
+	   	else
 		{
 		    prop->macro_regex[i] = NULL;
 		}
-
-
-	}    
+	}
+    }
+    	
+    /* save the commands and patterns */
+    for(i=0; i<=MAX_NUM_MACROS-1; i++) {
+        GConfValue *entry1, *entry2;
+	    
+	entry1 = gconf_value_new (GCONF_VALUE_STRING);
+	gconf_value_set_string (entry1, prop->macro_pattern[i]);
+	entry2 = gconf_value_new (GCONF_VALUE_STRING);
+	gconf_value_set_string (entry2, prop->macro_command[i]);
+	pattern_list = g_slist_append (pattern_list, entry1);
+	command_list = g_slist_append (command_list, entry2);	   
+    
+    }  
+    
+    macro_patterns = gconf_value_new (GCONF_VALUE_LIST);
+    macro_commands = gconf_value_new (GCONF_VALUE_LIST);
+    if (pattern_list && command_list) {
+        gconf_value_set_list_type (macro_patterns, GCONF_VALUE_STRING);
+        gconf_value_set_list (macro_patterns, pattern_list);
+        panel_applet_gconf_set_value (applet, "macro_patterns", macro_patterns, NULL);
+        gconf_value_set_list_type (macro_commands, GCONF_VALUE_STRING);
+        gconf_value_set_list (macro_commands, command_list);
+        panel_applet_gconf_set_value (applet, "macro_commands", macro_commands, NULL);        
+    }
 
     /* history */
     history = panel_applet_gconf_get_value (applet, "history", NULL);
@@ -324,43 +485,21 @@ load_session(MCData *mcdata)
             list = g_slist_next (list);
         }
     }
-    gnome_config_pop_prefix();
     
     return prop;
 }
 
-
-/* FIXME: port to gconf */
-void
-save_session(MCData *mcdata)
-{
-    PanelApplet *applet = mcdata->applet;
-    int i;
-    
-#if 0
-    for(i=0; i<=MAX_NUM_MACROS-1; i++)
-	{
-	    g_snprintf(section, sizeof(section), "mini_commander/macro_pattern_%.2u", i+1);
-	    gnome_config_set_string((gchar *) section, (gchar *) prop.macro_pattern[i]);
-
-	    g_snprintf(section, sizeof(section), "mini_commander/macro_command_%.2u", i+1);
-	    gnome_config_set_string((gchar *) section, (gchar *) prop.macro_command[i]);
-
-	    prop_tmp.macro_pattern[i] = (char *) NULL;
-	    prop_tmp.macro_command[i] = (char *) NULL;
-	}    
-
-#endif
-}
-
 static void
-phelp_cb (GtkWidget *w, gint tab, gpointer data)
+phelp_cb (void)
 {
-   #if 0 /* FIXME */
-        GnomeHelpMenuEntry help_entry = { "mini-commander_applet",
-                                          "index.html#MINI-COMMANDER-PREFS" };
-        gnome_help_display(NULL, &help_entry);
-   #endif
+    GError *error = NULL;
+    gnome_help_display ("command-line", "command-line-prefs-0", &error);
+
+    if (error) {
+        g_warning ("help error: %s\n", error->message);
+        g_error_free (error);
+        error = NULL;
+    }
 }
 
 static void
@@ -458,6 +597,35 @@ check_frame_toggled (GtkToggleButton *button, gpointer data)
     
 }
 
+static void
+check_theme_toggled (GtkToggleButton *button, gpointer data)
+{
+    MCData *mcdata = data;
+    PanelApplet *applet = mcdata->applet;
+    GtkWidget *fg_color_picker;
+    GtkWidget *bg_color_picker;
+    properties *prop;
+    gboolean toggled;
+    
+    prop = g_object_get_data (G_OBJECT (applet), "prop");
+    
+    toggled = gtk_toggle_button_get_active (button);
+    
+    if (toggled == prop->show_default_theme) 
+        return;
+        
+    prop->show_default_theme = toggled;
+    fg_color_picker = g_object_get_data ( G_OBJECT (applet), "fg_color_picker");
+    bg_color_picker = g_object_get_data ( G_OBJECT (applet), "bg_color_picker");
+    gtk_widget_set_sensitive( GTK_WIDGET (fg_color_picker), !(prop->show_default_theme));
+    gtk_widget_set_sensitive( GTK_WIDGET (bg_color_picker), !(prop->show_default_theme));
+    redraw_applet (mcdata);
+    panel_applet_gconf_set_bool (applet, "show_default_theme",
+                                 prop->show_default_theme,
+                                 NULL);
+
+}
+
 static void
 autocomplete_toggled (GtkToggleButton *button, gpointer data)
 {
@@ -483,6 +651,11 @@ static void
 response_cb (GtkDialog *dialog, gint it, gpointer data)
 {
     MCData *mcdata = data;
+
+    if(it == GTK_RESPONSE_HELP){
+        phelp_cb ();
+        return;
+    }
     
     gtk_widget_destroy (GTK_WIDGET (dialog));
     mcdata->properties_box = NULL;
@@ -502,7 +675,8 @@ properties_box(BonoboUIComponent *uic, gpointer data, const gchar *verbname)
     GtkWidget *vbox, *vbox1, *frame;
     GtkWidget *hbox;
     GtkWidget *table;
-    GtkWidget *check_time, *check_date, *check_handle, *check_frame, *check_auto_complete_history;
+    GtkWidget *check_time, *check_date, *check_handle, *check_frame, 
+              *check_auto_complete_history, *check_theme;
     GtkWidget *label;
     GtkWidget *entry;
     GtkWidget *spin;
@@ -517,10 +691,11 @@ properties_box(BonoboUIComponent *uic, gpointer data, const gchar *verbname)
 	return;
     }
     prop = g_object_get_data (G_OBJECT (applet), "prop");
-    mcdata->properties_box = gtk_dialog_new_with_buttons (_("Command Line Properties"), 
+    mcdata->properties_box = gtk_dialog_new_with_buttons (_("Command Line Preferences"), 
     						  NULL,
 						  GTK_DIALOG_DESTROY_WITH_PARENT,
 						  GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
+						  GTK_STOCK_HELP, GTK_RESPONSE_HELP,
 						  NULL);
     gtk_dialog_set_default_response (GTK_DIALOG (mcdata->properties_box), GTK_RESPONSE_CLOSE);
     gtk_window_set_default_size (GTK_WINDOW (mcdata->properties_box), 400, 300);
@@ -707,22 +882,36 @@ properties_box(BonoboUIComponent *uic, gpointer data, const gchar *verbname)
     gtk_container_set_border_width(GTK_CONTAINER(table), GNOME_PAD_SMALL);
     gtk_container_add(GTK_CONTAINER(frame), table);
 
+    /* default bg and fg theme */
+    check_theme = gtk_check_button_new_with_mnemonic("_Use default theme colors");
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_theme), prop->show_default_theme);
+    g_signal_connect (G_OBJECT (check_theme), "toggled",
+    		      G_CALLBACK (check_theme_toggled), mcdata); 
+    gtk_table_attach(GTK_TABLE(table),
+             check_theme,
+             0, 1,
+             0, 1,
+             0, 0,
+             0, 0);
+
     /* fg */
     label = gtk_label_new_with_mnemonic(_("Command line _foreground:"));
-    gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5);
+    gtk_misc_set_alignment (GTK_MISC (label), 1.0, 0.5); 
     gtk_table_attach(GTK_TABLE(table), 
 		     label,
 		     0, 1,
-		     0, 1,
+		     1, 2,
 		     GTK_FILL, 0,
 		     0, 0);
 
     color_picker = gnome_color_picker_new();
+    g_object_set_data ( G_OBJECT (applet), "fg_color_picker", color_picker);
     gnome_color_picker_set_i16(GNOME_COLOR_PICKER(color_picker),
 			       prop->cmd_line_color_fg_r, 
 			       prop->cmd_line_color_fg_g, 
 			       prop->cmd_line_color_fg_b, 
 			       0);
+    gtk_widget_set_sensitive ( GTK_WIDGET (color_picker), !(prop->show_default_theme));
     set_atk_relation(label, color_picker);
     gtk_signal_connect(GTK_OBJECT(color_picker),
 		       "color_set",
@@ -734,7 +923,7 @@ properties_box(BonoboUIComponent *uic, gpointer data, const gchar *verbname)
     gtk_table_attach(GTK_TABLE(table), 
 		     color_picker,
 		     1, 2,
-		     0, 1,
+		     1, 2,
 		     0, 0,
 		     0, 0);
     
@@ -745,15 +934,17 @@ properties_box(BonoboUIComponent *uic, gpointer data, const gchar *verbname)
     gtk_table_attach(GTK_TABLE(table), 
 		     label,
 		     0, 1,
-		     1, 2,
+		     2, 3,
 		     GTK_FILL, 0,
 		     0, 0);
     color_picker = gnome_color_picker_new();
+    g_object_set_data ( G_OBJECT (applet), "bg_color_picker", color_picker);
     gnome_color_picker_set_i16(GNOME_COLOR_PICKER(color_picker),
 			       prop->cmd_line_color_bg_r, 
 			       prop->cmd_line_color_bg_g, 
 			       prop->cmd_line_color_bg_b, 
 			       0);
+    gtk_widget_set_sensitive ( GTK_WIDGET (color_picker), !(prop->show_default_theme));
     set_atk_relation(label, color_picker);
     gtk_signal_connect(GTK_OBJECT(color_picker),
 		       "color_set",
@@ -762,7 +953,7 @@ properties_box(BonoboUIComponent *uic, gpointer data, const gchar *verbname)
     gtk_table_attach(GTK_TABLE(table), 
 		     color_picker,
 		     1, 2,
-		     1, 2,
+		     2, 3,
 		     0, 0,
 		     0, 0);
     
@@ -787,6 +978,10 @@ properties_box(BonoboUIComponent *uic, gpointer data, const gchar *verbname)
     gtk_container_set_border_width(GTK_CONTAINER(vbox1), GNOME_PAD_SMALL);
     /* gtk_container_add(GTK_CONTAINER(scrolled_window), vbox1); does not work */
     gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scrolled_window), vbox1);
+    gtk_container_set_focus_hadjustment (GTK_CONTAINER (vbox1),
+        gtk_scrolled_window_get_hadjustment (GTK_SCROLLED_WINDOW (scrolled_window)));
+    gtk_container_set_focus_vadjustment (GTK_CONTAINER (vbox1),
+        gtk_scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW (scrolled_window)));
 
     for(i=0; i < MAX_NUM_MACROS; i++)
 	{
@@ -794,8 +989,8 @@ properties_box(BonoboUIComponent *uic, gpointer data, const gchar *verbname)
   	    gtk_box_pack_start(GTK_BOX(vbox1), hbox, TRUE, TRUE, 0);   
 	    
 	    /* prefix */    
-	    g_snprintf(text_label, sizeof(text_label), _("Regex %.2d:"), i+1);
-	    label = gtk_label_new(text_label);
+	    g_snprintf(text_label, sizeof(text_label), _("Regex _%.2d:"), i+1);
+	    label = gtk_label_new_with_mnemonic(text_label);
 	    gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, TRUE, 0);
 	    
 	    entry = gtk_entry_new_with_max_length(MAX_MACRO_PATTERN_LENGTH);
@@ -803,28 +998,32 @@ properties_box(BonoboUIComponent *uic, gpointer data, const gchar *verbname)
 	    if (prop->macro_pattern[i] != (gchar *) NULL)
 		gtk_entry_set_text(GTK_ENTRY(entry), (gchar *) prop->macro_pattern[i]);
 	    set_atk_relation(label, entry);
-	    #ifdef FIXME
+	    g_object_set_data (G_OBJECT (entry), "prop", prop);
+	    g_object_set_data (G_OBJECT (entry), "applet", applet);
+	    #if 1
 	    gtk_signal_connect(GTK_OBJECT(entry),
-			       "changed",
-			       GTK_SIGNAL_FUNC(entry_changed_signal),
-			       &prop->macro_pattern[i]);
+			       "focus_out_event",
+			       GTK_SIGNAL_FUNC(pattern_entry_changed_signal),
+			       GINT_TO_POINTER (i));
 	    #endif
 	    gtk_box_pack_start(GTK_BOX(hbox), entry, FALSE, TRUE, 0);
 	    
 	    /* command */
-	    g_snprintf(text_label, sizeof(text_label), _("   Macro %.2d:"), i+1);
-	    label = gtk_label_new(text_label);
+	    g_snprintf(text_label, sizeof(text_label), _("   Macro _%.2d:"), i+1);
+	    label = gtk_label_new_with_mnemonic(text_label);
 	    gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, TRUE, 0);
 	    
 	    entry = gtk_entry_new_with_max_length(MAX_COMMAND_LENGTH);
 	    if (prop->macro_command[i] != (gchar *) NULL)
 		gtk_entry_set_text(GTK_ENTRY(entry), prop->macro_command[i]);
 	    set_atk_relation(label, entry);
-	    #ifdef FIXME
+	    g_object_set_data (G_OBJECT (entry), "prop", prop);
+	    g_object_set_data (G_OBJECT (entry), "applet", applet);
+	    #if 1
 	    gtk_signal_connect(GTK_OBJECT(entry),
-			       "changed",
-			       GTK_SIGNAL_FUNC(entry_changed_signal),
-			       &prop_tmp.macro_command[i]);
+			       "focus_out_event",
+			       GTK_SIGNAL_FUNC(command_entry_changed_signal),
+			       GINT_TO_POINTER (i));
 	    #endif
 	    gtk_box_pack_start(GTK_BOX(hbox), entry, TRUE, TRUE, 0);
 	}
