@@ -90,7 +90,7 @@ static gboolean panel_menu_accept_entry_drop (PanelMenu *panel_menu,
 
 static gint applet_button_press_cb (GtkWidget *widget, GdkEventButton *event,
 				    PanelMenu *panel_menu);
-static gint applet_destroy_cb (GtkWidget *widget, PanelMenu *panel_menu);
+static void applet_destroy_cb (GtkWidget *widget, PanelMenu *panel_menu);
 static void applet_change_orientation_cb (PanelApplet *applet,
 					  PanelAppletOrient orient,
 					  PanelMenu *panel_menu);
@@ -124,13 +124,13 @@ static const BonoboUIVerb applet_menu_verbs[] = {
 static const char applet_menu_xml[] =
 	"<popup name=\"button3\">\n"
 	"   <placeholder name=\"ChildMerge\"/>\n"
-	"   <menuitem name=\"Add\" verb=\"Add\" _label=\"Add...\"\n"
+	"   <menuitem name=\"Add\" verb=\"Add\" _label=\"_Add...\"\n"
 	"             pixtype=\"stock\" pixname=\"gtk-add\"/>\n"
-	"   <menuitem name=\"Properties Item\" verb=\"Properties\" _label=\"Properties ...\"\n"
+	"   <menuitem name=\"Properties Item\" verb=\"Properties\" _label=\"_Preferences...\"\n"
 	"             pixtype=\"stock\" pixname=\"gtk-properties\"/>\n"
-	"   <menuitem name=\"Help Item\" verb=\"Help\" _label=\"Help\"\n"
+	"   <menuitem name=\"Help Item\" verb=\"Help\" _label=\"_Help\"\n"
 	"             pixtype=\"stock\" pixname=\"gtk-help\"/>\n"
-	"   <menuitem name=\"About Item\" verb=\"About\" _label=\"About ...\"\n"
+	"   <menuitem name=\"About Item\" verb=\"About\" _label=\"_About ...\"\n"
 	"             pixtype=\"stock\" pixname=\"gnome-stock-about\"/>\n"
 	"</popup>\n";
 
@@ -579,7 +579,7 @@ applet_button_press_cb (GtkWidget *widget, GdkEventButton *event,
 	return TRUE;
 }
 
-static gint
+static void
 applet_destroy_cb (GtkWidget *widget, PanelMenu *panel_menu)
 {
 	PanelMenuEntry *entry;
@@ -600,8 +600,10 @@ applet_destroy_cb (GtkWidget *widget, PanelMenu *panel_menu)
 		entry = (PanelMenuEntry *) cur->data;
 		panel_menu_common_call_entry_destroy (entry);
 	}
-	if (panel_menu->entries)
+	if (panel_menu->entries) {
 		g_list_free (panel_menu->entries);
+	}
+	panel_menu_pixbuf_exit ();
 	g_free (panel_menu);
 }
 
@@ -663,6 +665,12 @@ applet_about_cb (BonoboUIComponent *uic, PanelMenu *panel_menu,
 		NULL
 	};
 
+	const gchar *documenters[] = {
+		NULL
+	};
+
+	const gchar *translator_credits = _("translator_credits");
+
 	if (about != NULL) {
 		gtk_widget_show (about);
 		gtk_window_present (GTK_WINDOW (about));
@@ -679,8 +687,12 @@ applet_about_cb (BonoboUIComponent *uic, PanelMenu *panel_menu,
 		pixbuf = gdk_pixbuf_new_from_file (file, NULL);
 	}
 
-	about = gnome_about_new (_("PanelMenu Applet"), "0.0.1", _("(c) 2001 Chris Phelps"), _("The Panel Menu Applet allows you to display customized menubars on your panels."), authors, NULL,	/* documenters */
-				 NULL,	/* translator_credits */
+	about = gnome_about_new (_("Menu Bar"), "0.0.1",
+				 _("(c) 2001 Chris Phelps"),
+				 _("The Menu Bar allows you to display customized menubars on your panels."),
+				 authors,
+				 documenters,
+				 strcmp (translator_credits, "translator_credits") != 0 ? translator_credits : NULL,
 				 pixbuf);
 
 	gtk_window_set_wmclass (GTK_WINDOW (about), "panel-menu", "PanelMenu");
